@@ -1,9 +1,9 @@
 #!/usr/bin/env perl
-# $Id: fmtutil.pl 39813 2016-02-22 05:25:37Z preining $
+# $Id: fmtutil.pl 53340 2020-01-06 23:55:32Z preining $
 # fmtutil - utility to maintain format files.
 # (Maintained in TeX Live:Master/texmf-dist/scripts/texlive.)
 # 
-# Copyright 2014-2016 Norbert Preining
+# Copyright 2014-2020 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
@@ -24,11 +24,11 @@ BEGIN {
   TeX::Update->import();
 }
 
-my $svnid = '$Id: fmtutil.pl 39813 2016-02-22 05:25:37Z preining $';
-my $lastchdate = '$Date: 2016-02-22 06:25:37 +0100 (Mon, 22 Feb 2016) $';
+my $svnid = '$Id: fmtutil.pl 53340 2020-01-06 23:55:32Z preining $';
+my $lastchdate = '$Date: 2020-01-07 00:55:32 +0100 (Tue, 07 Jan 2020) $';
 $lastchdate =~ s/^\$Date:\s*//;
 $lastchdate =~ s/ \(.*$//;
-my $svnrev = '$Revision: 39813 $';
+my $svnrev = '$Revision: 53340 $';
 $svnrev =~ s/^\$Revision:\s*//;
 $svnrev =~ s/\s*\$$//;
 my $version = "r$svnrev ($lastchdate)";
@@ -37,6 +37,7 @@ use strict;
 use Getopt::Long qw(:config no_autoabbrev ignore_case_always);
 use File::Basename;
 use File::Copy;
+use File::Spec;
 use Cwd;
 
 # don't import anything automatically, this requires us to explicitly
@@ -44,6 +45,8 @@ use Cwd;
 # find and if necessary remove references to TLUtils
 use TeXLive::TLUtils qw();
 
+require TeXLive::TLWinGoo if TeXLive::TLUtils::win32;
+
 # numerical constants
 my $FMT_NOTSELECTED = 0;
 my $FMT_DISABLED    = 1;
@@ -59,6 +62,9 @@ my @deferred_stdout;
 
 (our $prg = basename($0)) =~ s/\.pl$//;
 
+# make sure that the main binary path is available at the front
+TeXLive::TLUtils::prepend_own_path();
+
 # sudo sometimes does not reset the home dir of root, check on that
 # see more comments at the definition of the function itself
 # this function checks by itself whether it is running on windows or not
@@ -88,7 +94,9 @@ if (win32()) {
 our $texmfconfig = $TEXMFCONFIG;
 our $texmfvar    = $TEXMFVAR;
 our $alldata;
-our %opts = ( quiet => 0 );
+# command line options with defaults
+# 20160623 - switch to turn on strict mode
+our %opts = ( quiet => 0 , strict => 1 );
 
 # make a list of all the commands (as opposed to options), so we can
 # reasonably check for multiple commands being (erroneously) given.
@@ -107,6 +115,7 @@ my @cmdline_cmds = (  # in same order as help message
 
 our @cmdline_options = (  # in same order as help message
   "sys",
+  "user",
   "cnffile=s@", 
   "fmtdir=s",
   "no-engine-subdir",
@@ -114,7 +123,7 @@ our @cmdline_options = (  # in same order as help message
   "no-error-if-no-format",
   "nohash",
   "recorder",
-  "strict",
+  "strict!",
   "quiet|silent|q",
   "catcfg",
   "dolinks",
@@ -146,8 +155,14 @@ sub main {
     # first generated filename after successful generation to stdout then
     # (and nothing else), since kpathsea can only deal with one.
     $mktexfmtMode = 1;
+
+    # TODO TODO
+    # which mode are we running in?
+    # what happens if root runs mktexfmt?
+    $opts{'user'} = 1;
+
     GetOptions ( "help" => \$opts{'help'}, "version" => \$opts{'version'} )
-        or die "Unknown option in mktexfmt command line arguments\n";
+      || die "$prg: Unknown option in mktexfmt command line arguments.\n";
     if ($ARGV[0]) {
       if ($ARGV[0] =~ m/^(.*)\.(fmt|mem|base?)$/) {
         $opts{'byfmt'} = $1;
@@ -193,7 +208,10 @@ sub main {
   }
   
   # these two functions should go to TLUtils (for use in updmap)
-  check_hidden_sysmode();
+  ($texmfconfig, $texmfvar) = 
+    TeXLive::TLUtils::setup_sys_user_mode($prg, \%opts,
+      $TEXMFCONFIG, $TEXMFSYSCONFIG, $TEXMFVAR, $TEXMFSYSVAR);
+
   determine_config_files("fmtutil.cnf");
   my $changes_config_file = $alldata->{'changes_config'};
   # we do changes always in the used config file with the highest priority
@@ -321,10 +339,15 @@ sub callback_build_formats {
   my $tmpdir;
   if (win32()) {
     my $foo;
+    my $tmp_deflt = File::Spec->tmpdir;
     for my $i (1..5) {
-      $foo = "$texmfvar/temp.$$." . int(rand(1000000));
+      # $foo = "$texmfvar/temp.$$." . int(rand(1000000));
+      $foo = (($texmfvar =~ m!^//!) ? $tmp_deflt : $texmfvar)
+        . "/temp.$$." . int(rand(1000000));
       if (! -d $foo) {
-        if (mkdir($foo)) {
+        TeXLive::TLUtils::mkdirhier($foo);
+        sleep 1;
+        if (-d $foo) {
           $tmpdir = $foo;
           last;
         }
@@ -333,6 +356,10 @@ sub callback_build_formats {
     if (! $tmpdir) {
       die "Cannot get a temporary directory after five iterations ... sorry!";
     }
+    if ($texmfvar =~ m!^//!) {
+      # used File::Spec->tmpdir; fix permissions
+      TeXLive::TLWinGoo::maybe_make_ro ($tmpdir);
+    }
   } else {
     $tmpdir = File::Temp::tempdir(CLEANUP => 1);
   }
@@ -349,7 +376,7 @@ sub callback_build_formats {
   # for safety, check again
   die "abs_path failed, strange: $!" if !$opts{'fmtdir'};
   print_info("writing formats under $opts{fmtdir}\n"); # report
-   
+
   # code taken over from the original shell script for KPSE_DOT etc
   my $thisdir = cwd();
   $ENV{'KPSE_DOT'} = $thisdir;
@@ -409,20 +436,23 @@ sub callback_build_formats {
     }
   }
 
-  # if the user asked to rebuild something, but we did nothing, report.
-  if ($err + $suc == 0) {
-    print_info("did not find entry for $what=$whatarg, skipped\n");
+  # if the user asked to rebuild something, but we did nothing, report
+  # unless we tried to rebuild only missing formats.
+  if ($what ne "missing") {
+    if ($err + $suc == 0) {
+      print_info("did not find entry for $what=$whatarg, skipped\n");
+    }
   }
   my $stdo = ($mktexfmtMode ? \*STDERR : \*STDOUT);
   for (@deferred_stdout) { print $stdo $_; }
   for (@deferred_stderr) { print STDERR $_; }
   #
-  print_info("Disabled formats: $disabled\n")        if ($disabled);
-  print_info("Successfully rebuilt formats: $suc\n") if ($suc);
-  print_info("Not selected formats: $nobuild\n")     if ($nobuild);
-  print_info("Not available formats: $notavail\n")   if ($notavail);
-  print_info("Failed to build: $err (@err)\n")       if ($err);
-  print_info("Total formats: $total\n");
+  print_info("disabled formats: $disabled\n")        if ($disabled);
+  print_info("successfully rebuilt formats: $suc\n") if ($suc);
+  print_info("not selected formats: $nobuild\n")     if ($nobuild);
+  print_info("not available formats: $notavail\n")   if ($notavail);
+  print_info("failed to build: $err (@err)\n")       if ($err);
+  print_info("total formats: $total\n");
   chdir($thisdir) || warn "chdir($thisdir) failed: $!";
   if (win32()) {
     # try to remove the tmpdir with all files
@@ -440,12 +470,15 @@ sub select_and_rebuild_format {
   my ($fmt, $eng, $what, $whatarg) = @_;
   return $FMT_DISABLED
       if ($alldata->{'merged'}{$fmt}{$eng}{'status'} eq 'disabled');
+
+  my ($kpsefmt, $destdir, $fmtfile, $logfile) = compute_format_destination($fmt, $eng);
+
   my $doit = 0;
   # we just identify 'all', 'refresh', 'missing'
   # I don't see much point in keeping all of them
   $doit = 1 if ($what eq 'all');
-  $doit = 1 if ($what eq 'refresh');
-  $doit = 1 if ($what eq 'missing');
+  $doit = 1 if ($what eq 'refresh' && -r "$destdir/$fmtfile");
+  $doit = 1 if ($what eq 'missing' && ! -r "$destdir/$fmtfile");
   $doit = 1 if ($what eq 'byengine' && $eng eq $whatarg);
   $doit = 1 if ($what eq 'byfmt' && $fmt eq $whatarg);
   # TODO
@@ -483,18 +516,54 @@ sub select_and_rebuild_format {
     }
   }
   if ($doit) {
-    return rebuild_one_format($fmt,$eng);
+    return rebuild_one_format($fmt,$eng,$kpsefmt,$destdir,$fmtfile,$logfile);
   } else {
     return $FMT_NOTSELECTED;
   }
 }
 
+#  compute_format_destination
+# takes fmt/eng and returns the locations where format and log files
+# should be saved, that is, a list: (dump file full path, log file full path)
+# 
+sub compute_format_destination {
+  my ($fmt, $eng) = @_;
+  my $enginedir;
+  my $fmtfile = $fmt;
+  my $kpsefmt;
+  my $destdir;
+
+  if ($eng eq "mpost") {
+    $fmtfile .= ".mem" ;
+    $kpsefmt = "mp" ;
+    $enginedir = "metapost"; # the directory, not the executable
+  } elsif ($eng =~ m/^mf(lua(jit)?)?(w|-nowin)?$/) {
+    $fmtfile .= ".base" ;
+    $kpsefmt = "mf" ;
+    $enginedir = "metafont" ;
+  } else {
+    $fmtfile .= ".fmt" ;
+    $kpsefmt = "tex" ;
+    $enginedir = $eng;
+    # strip final -dev from enginedir to support engines like luatex-dev
+    $enginedir =~ s/-dev$//;
+  }
+  if ($opts{'no-engine-subdir'}) {
+    $destdir = $opts{'fmtdir'};
+  } else {
+    $destdir = "$opts{'fmtdir'}/$enginedir";
+  }
+  return($kpsefmt, $destdir, $fmtfile, "$fmt.log");
+}
+
+
 #  rebuild_one_format
-# takes fmt/eng and rebuilds it, irrelevant of any setting
+# takes fmt/eng and rebuilds it, irrelevant of any setting;
+# copies generated log file
 # return value FMT_*
 #
 sub rebuild_one_format {
-  my ($fmt, $eng) = @_;
+  my ($fmt,$eng,$kpsefmt,$destdir,$fmtfile,$logfile) = @_;
   print_info("--- remaking $fmt with $eng\n");
 
   # get variables
@@ -502,16 +571,13 @@ sub rebuild_one_format {
   my $addargs = $alldata->{'merged'}{$fmt}{$eng}{'args'};
 
   # running parameters
-  my $texengine;
   my $jobswitch = "-jobname=$fmt";
   my $prgswitch = "-progname=" ;
   my $recorderswitch = ($opts{'recorder'} ? "-recorder" : "");
-  my $fmtfile = $fmt;
-  my $kpsefmt;
   my $pool;
   my $tcx = "";
   my $tcxflag = "";
-  my $localpool=0;
+  my $localpool = 0;
   my $texargs;
 
   unlink glob "*.pool";
@@ -530,20 +596,6 @@ sub rebuild_one_format {
   elsif ($fmt =~ m/^cont-..$/) { $prgswitch .= "context"; }
   else                         { $prgswitch .= $fmt; }
 
-  if ($eng eq "mpost") { 
-    $fmtfile .= ".mem" ; 
-    $kpsefmt = "mp" ; 
-    $texengine = "metapost"; # the directory, not the executable
-  } elsif ($eng =~ m/^mf(w|-nowin)?$/) {
-    $fmtfile .= ".base" ; 
-    $kpsefmt = "mf" ; 
-    $texengine = "metafont";
-  } else {
-    $fmtfile .= ".fmt" ; 
-    $kpsefmt = "tex" ; 
-    $texengine = $eng;
-  }
-  
   # check for existence of ini file before doing anything else
   if (system("kpsewhich -progname=$fmt -format=$kpsefmt $inifile >$nul 2>&1") != 0) {
     # we didn't find the ini file, skip
@@ -552,7 +604,15 @@ sub rebuild_one_format {
     # all activated formats are also buildable, thus return failure.
     return $FMT_FAILURE;
   }
-  
+
+  #
+  # If the 4th field in fmtutil.cnf contains
+  #  -progname=...
+  # then we do not add our own progname!
+  if ($addargs =~ /-progname=/) {
+    $prgswitch = '';
+  }
+
   # NLS support
   #   Example (for fmtutil.cnf):
   #     mex-pl tex mexconf.tex nls=tex-pl,il2-pl mex.ini
@@ -566,11 +626,11 @@ sub rebuild_one_format {
     $texargs = $addargs;
   }
   if ($pool) {
-    chomp ( my $poolfile = `kpsewhich -progname=$eng $pool.poo 2>$nul` );
+    chomp (my $poolfile = `kpsewhich -progname=$eng $pool.poo 2>$nul`);
     if ($poolfile && -f $poolfile) {
       print_verbose("attempting to create localized format "
                     . "using pool=$pool and tcx=$tcx.\n");
-      File::Copy($poolfile, "$eng.pool");
+      File::Copy::copy($poolfile, "$eng.pool");
       $tcxflag = "-translate-file=$tcx" if ($tcx);
       $localpool = 1;
     }
@@ -598,7 +658,7 @@ sub rebuild_one_format {
         ",$opts{'no-error-if-no-engine'}," =~ m/,$eng,/) {
       return $FMT_NOTAVAIL;
     } else {
-      print_deferred_error("not building $fmt due to missing engine $eng.\n");
+      print_deferred_error("not building $fmt due to missing engine: $eng\n");
       return $FMT_FAILURE;
     }
   }
@@ -617,17 +677,33 @@ sub rebuild_one_format {
     $cmdline .= " >&2" if $mktexfmtMode;
     $cmdline .= " <$nul";
     my $retval = system($cmdline);
+    
+    # report error if it failed.
     if ($retval != 0) {
       $retval /= 256 if ($retval > 0);
-      print_deferred_error("running \`$cmdline' return status $retval\n");
-      #
-      # original shell script did *not* check the return value
-      # we keep this behavior, but add an option --strict that
-      # errors out on all failures.
-      if ($opts{'strict'}) {
-        print_deferred_error("return error due to options --strict\n");
-        return $FMT_FAILURE;
-      }
+      print_deferred_error("running \`$cmdline' return status: $retval\n");
+    }
+
+    # Copy the log file after the program is run, so that the log file
+    # is available to inspect even on failure. So we need the dest dir tree.
+    TeXLive::TLUtils::mkdirhier($destdir);
+    #
+    # Here and in the following we use copy instead of move
+    # to make sure that in SElinux enabled cases the rules of
+    # the destination directory are applied.
+    # See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=900580
+    if (File::Copy::copy($logfile, "$destdir/$logfile")) {
+      print_info("log file copied to: $destdir/$logfile\n");
+    } else {
+      print_deferred_error("cannot copy log $logfile to: $destdir\n");
+    }
+
+    # original shell script did *not* check the return value
+    # we keep this behavior, but add an option --strict that
+    # errors out on all failures.
+    if ($retval != 0 && $opts{'strict'}) {
+      print_deferred_error("returning error due to option --strict\n");
+      return $FMT_FAILURE;
     }
 
     if ($localpool) {
@@ -637,7 +713,6 @@ sub rebuild_one_format {
         delete $ENV{'TEXPOOL'};
       }
     }
-
   }
 
   # check and install of fmt and log files
@@ -646,42 +721,31 @@ sub rebuild_one_format {
     return $FMT_FAILURE;
   }
 
-  if (! -f "$fmt.log") {
+  if (! -f $logfile) {
     print_deferred_error("no log file generated for $fmt/$eng, strange\n");
     return $FMT_FAILURE;
   }
 
-  open (LOGFILE, "<$fmt.log")
-    || print_deferred_warning("cannot open $fmt.log, strange: $!\n");
+  open (LOGFILE, "<$logfile")
+    || print_deferred_warning("cannot open $logfile, strange: $!\n");
   my @logfile = <LOGFILE>;
   close LOGFILE;
   if (grep(/^!/, @logfile) > 0) {
     print_deferred_error("\`$cmdline' had errors.\n");
   }
 
-  my $fulldestdir;
-  if ($opts{'no-engine-subdir'}) {
-    $fulldestdir = $opts{'fmtdir'};
-  } else {
-    $fulldestdir = "$opts{'fmtdir'}/$texengine";
-  }
-  TeXLive::TLUtils::mkdirhier($fulldestdir);
-  
-  if (!File::Copy::move( "$fmt.log", "$fulldestdir/$fmt.log")) {
-    print_deferred_error("Cannot move $fmt.log to $fulldestdir.\n");
-  }
   if ($opts{'recorder'}) {
-    # the recorder output is used by check-fmttriggers to determine
+    # the recorder output is used by tl-check-fmttriggers to determine
     # package dependencies for each format.  Unfortunately omega-based
     # engines gratuitiously changed the extension from .fls to .ofl.
     my $recfile = $fmt . ($fmt =~ m/^(aleph|lamed)$/ ? ".ofl" : ".fls");
-    if (!File::Copy::move( $recfile, "$fulldestdir/$recfile")) {
-      print_deferred_error("Cannot move $recfile to $fulldestdir.\n");
+    if (!File::Copy::copy($recfile, "$destdir/$recfile")) {
+      print_deferred_error("cannot copy recorder $recfile to: $destdir\n");
     }
   }
 
-  my $destfile = "$fulldestdir/$fmtfile";
-  if (File::Copy::move( $fmtfile, $destfile )) {
+  my $destfile = "$destdir/$fmtfile";
+  if (File::Copy::copy($fmtfile, $destfile )) {
     print_info("$destfile installed.\n");
     #
     # original fmtutil.sh did some magic trick for mplib-luatex.mem
@@ -728,10 +792,10 @@ sub rebuild_one_format {
     return $FMT_SUCCESS;
 
   } else {
-    print_deferred_error("Cannot move $fmtfile to $destfile.\n");
+    print_deferred_error("cannot copy format $fmtfile to: $destfile\n");
     if (-f $destfile) {
       # remove the empty file possibly left over if near-full file system.
-      print_verbose("Removing partial file after move failure: $destfile\n");
+      print_verbose("removing partial file after copy failure: $destfile\n");
       unlink($destfile)
         || print_deferred_error("unlink($destfile) failed: $!\n");
     }
@@ -853,6 +917,8 @@ sub callback_list_cfg {
   @lines = map { $_->[1] } sort { $a->[0] cmp $b->[0] } @lines;
   print "List of all formats:\n";
   print @lines;
+  
+  return @lines == 0; # only return failure if no formats.
 }
 
 
@@ -894,27 +960,46 @@ sub read_fmtutil_file {
   my $fn = shift;
   open(FN, "<$fn") || die "Cannot read $fn: $!";
   #
-  # we count lines from 0 ..!!!!
+  # we count lines from 0 ..!!!!?
   my $i = -1;
+  my $printline = 0; # but not in error messages
   my @lines = <FN>;
   chomp(@lines);
   $alldata->{'fmtutil'}{$fn}{'lines'} = [ @lines ];
   close(FN) || warn("$prg: Cannot close $fn: $!");
   for (@lines) {
     $i++;
+    $printline++;
     chomp;
+    my $orig_line = $_;
     next if /^\s*#?\s*$/; # ignore empty and all-blank and just-# lines
     next if /^\s*#[^!]/;  # ignore whole-line comment that is not a disable
     s/#[^!].*//;          # remove within-line comment that is not a disable
     s/#$//;               # remove # at end of line
     my ($a,$b,$c,@rest) = split (' '); # special split rule, leading ws ign
+    if (! $b) { # as in: "somefmt"
+      print_warning("no engine specified for format $a, ignoring "
+                    . "(file $fn, line $printline)\n");
+      next;
+    }
+    if (! $c) { # as in: "somefmt someeng"
+      print_warning("no pattern argument specified for $a/$b, ignoring line: "
+                    . "$orig_line (file $fn, line $printline)\n");
+      next;
+    }
+    if (@rest == 0) { # as in: "somefmt someeng somepat"
+      print_warning("no inifile argument(s) specified for $a/$b, ignoring line: "
+                    . "$orig_line (file $fn, line $printline)\n");
+      next;
+    }
     my $disabled = 0;
     if ($a eq "#!") {
-      # we cannot determine whether a line is a proper fmtline or
-      # not, so we have to assume that it is
+      # we cannot feasibly determine whether a line is a proper fmtline or
+      # not, so we have to assume that it is as long as we have four args.
       my $d = shift @rest;
       if (!defined($d)) {
-        print_warning("apparently not a real disable line, ignored: $_\n");
+        print_warning("apparently not a real disable line, ignoring: "
+                      . "$orig_line (file $fn, line $printline)\n");
         next;
       } else {
         $disabled = 1;
@@ -940,32 +1025,6 @@ sub read_fmtutil_file {
 # and also be reused in updmap.pl!!!
 
 
-sub check_hidden_sysmode {
-  #
-  # check if we are in *hidden* sys mode, in which case we switch
-  # to sys mode
-  # Nowdays we use -sys switch instead of simply overriding TEXMFVAR
-  # and TEXMFCONFIG
-  # This is used to warn users when they run updmap in usermode the first time.
-  # But it might happen that this script is called via another wrapper that
-  # sets TEXMFCONFIG and TEXMFVAR, and does not pass on the -sys option.
-  # for this case we check whether the SYS and non-SYS variants agree,
-  # and if, then switch to sys mode (with a warning)
-  if (($TEXMFSYSCONFIG eq $TEXMFCONFIG) && ($TEXMFSYSVAR eq $TEXMFVAR)) {
-    if (!$opts{'sys'}) {
-      print_warning("hidden sys mode found, switching to sys mode.\n");
-      $opts{'sys'} = 1;
-    }
-  }
-
-  if ($opts{'sys'}) {
-    # we are running as updmap-sys, make sure that the right tree is used
-    $texmfconfig = $TEXMFSYSCONFIG;
-    $texmfvar    = $TEXMFSYSVAR;
-  }
-}
-
-
 # sets global $alldata->{'changes_config'} to the config file to be
 # changed if requested.
 #
@@ -981,7 +1040,18 @@ sub determine_config_files {
     my @tmp;
     for my $f (@{$opts{'cnffile'}}) {
       if (! -f $f) {
-        die "$prg: Config file \"$f\" not found.";
+        # if $f is a pure file name, that is dirname $f == ".",
+        # then try to find it via kpsewhich
+        if (dirname($f) eq ".") {
+          chomp(my $kpfile = `kpsewhich $f`);
+          if ($kpfile ne "") {
+            $f = $kpfile;
+          } else {
+            die "$prg: Config file \"$f\" cannot be found via kpsewhich";
+          }
+        } else {
+          die "$prg: Config file \"$f\" not found";
+        }
       }
       push @tmp, (win32() ? lc($f) : $f);
     }
@@ -1240,12 +1310,13 @@ sub version {
 
 sub help {
   my $usage = <<"EOF";
-Usage: $prg     [OPTION] ... [COMMAND]
-   or: $prg-sys [OPTION] ... [COMMAND]
-   or: mktexfmt FORMAT.fmt|BASE.base|MEM.mem|FMTNAME.EXT
+Usage: $prg      [-user|-sys] [OPTION] ... [COMMAND]
+   or: $prg-sys  [OPTION] ... [COMMAND]
+   or: $prg-user [OPTION] ... [COMMAND]
+   or: mktexfmt  FORMAT.fmt|BASE.base|FMTNAME.EXT
 
-Rebuild and manage TeX fmts, Metafont bases and MetaPost mems,
-collectively called "formats" here.
+Rebuild and manage TeX fmts and Metafont bases, collectively called
+"formats" here. (MetaPost no longer uses the past-equivalent "mems".)
 
 If the command name ends in mktexfmt, only one format can be created.
 The only options supported are --help and --version, and the command
@@ -1257,8 +1328,12 @@ If not operating in mktexfmt mode, exactly one command must be given,
 extensions should generally not be specified, no non-option arguments
 are allowed, and multiple formats can be generated, as follows.
 
+By default, the return status is zero if all formats requested are
+successfully built, else nonzero.
+
 Options:
-  --sys                   use TEXMFSYS{VAR,CONFIG} instead of TEXMF{VAR,CONFIG}
+  --sys                   use TEXMFSYS{VAR,CONFIG}
+  --user                  use TEXMF{VAR,CONFIG}
   --cnffile FILE          read FILE instead of fmtutil.cnf
                            (can be given multiple times, in which case
                            all the files are used)
@@ -1268,9 +1343,9 @@ Options:
   --no-error-if-no-engine=ENGINE1,ENGINE2,...
                           exit successfully even if a required engine
                            is missing, if it is included in the list.
+  --no-strict             exit successfully even if a format fails to build
   --nohash                don't update ls-R files
   --recorder              pass the -recorder option and save .fls files
-  --strict                exit with bad status if a format fails to build
   --quiet                 be silent
   --catcfg                (does nothing, exists for compatibility)
   --dolinks               (does nothing, exists for compatibility)
@@ -1299,7 +1374,9 @@ Explanation of trees and files normally used:
   If --cnffile is specified on the command line (possibly multiple
   times), its value(s) are used.  Otherwise, fmtutil reads all the
   fmtutil.cnf files found by running \`kpsewhich -all fmtutil.cnf', in the
-  order returned by kpsewhich.
+  order returned by kpsewhich. Files passed in via --cnffile are
+  first tried to be loaded directly, and if not found and the file names
+  don't contain directory parts, are searched via kpsewhich.
 
   In any case, if multiple fmtutil.cnf files are found, all the format
   definitions found in all the fmtutil.cnf files are merged.
@@ -1314,7 +1391,7 @@ Explanation of trees and files normally used:
   TEXMFLOCAL     \$TEXLIVE/texmf-local/web2c/fmtutil.cnf
   TEXMFDIST      \$TEXLIVE/YYYY/texmf-dist/web2c/fmtutil.cnf
 
-  For fmtutil:
+  For fmtutil-user:
   TEXMFCONFIG    \$HOME/.texliveYYYY/texmf-config/web2c/fmtutil.cnf
   TEXMFVAR       \$HOME/.texliveYYYY/texmf-var/web2c/fmtutil.cnf
   TEXMFHOME      \$HOME/texmf/web2c/fmtutil.cnf
@@ -1382,16 +1459,25 @@ Disabling formats:
   --no-error-if-no-engine option exists, since luajittex cannot be
   compiled on all platforms.)
 
-fmtutil vs. fmtutil-sys (fmtutil --sys):
+fmtutil-user (fmtutil -user) vs. fmtutil-sys (fmtutil -sys):
 
-  When fmtutil-sys is run or the command line option --sys is used, 
+  When fmtutil-sys is run or the command line option -sys is used, 
   TEXMFSYSCONFIG and TEXMFSYSVAR are used instead of TEXMFCONFIG and 
   TEXMFVAR, respectively.  This is the primary difference between 
-  fmtutil-sys and fmtutil.
+  fmtutil-sys and fmtutil-user.
+
+  See http://tug.org/texlive/scripts-sys-user.html for details.
 
   Other locations may be used if you give them on the command line, or
   these trees don't exist, or you are not using the original TeX Live.
 
+Supporting development binaries
+
+  If an engine name ends with "-dev", formats are created in
+  the respective directory with the -dev stripped.  This allows for
+  easily running development binaries in parallel with the released
+  binaries.
+
 Report bugs to: tex-live\@tug.org
 TeX Live home page: <http://tug.org/texlive/>
 EOF
