@@ -8,8 +8,8 @@
 
 local ProvidesLuaModule = { 
     name          = "luaotfload-loaders",
-    version       = "2.97",       --TAGVERSION
-    date          = "2019-05-18", --TAGDATE
+    version       = "3.12",       --TAGVERSION
+    date          = "2020-02-02", --TAGDATE
     description   = "luaotfload submodule / callback handling",
     license       = "GPL v2.0"
 }
@@ -27,17 +27,8 @@ if not luaotfload then error "this module requires Luaotfload" end
 local logreport = luaotfload.log and luaotfload.log.report or print
 
 local lua_reader = function (specification)
-  local fullname = specification.filename or ""
-  if fullname == "" then
-    local forced = specification.forced or ""
-    if forced ~= "" then
-      fullname = specification.name .. "." .. forced
-    else
-      fullname = specification.name
-    end
-  end
-  local fullname = resolvers.findfile (fullname) or ""
-  if fullname ~= "" then
+  local fullname = specification.resolved
+  if fullname then
     local loader = loadfile (fullname)
     loader = loader and loader ()
     return loader and loader (specification)
@@ -129,20 +120,35 @@ end
 
 local definers --- (string, spec -> size -> id -> tmfdata) hash_t
 do
-  local read = fonts.definers.read
+  local ctx_read = fonts.definers.read
+  local register = fonts.definers.register
+  local function read(specification, size, id)
+    local tfmdata = ctx_read(specification, size, id)
+    if not tfmdata or id or tonumber(tfmdata) then
+      return tfmdata
+    end
+    id = font.define(tfmdata)
+    register(tfmdata, id)
+    return id
+  end
 
   local patch = function (specification, size, id)
-    local fontdata = read (specification, size, id)
+    local fontdata = ctx_read (specification, size, id)
 ----if not fontdata then not_found_msg (specification, size, id) end
     if type (fontdata) == "table" and fontdata.encodingbytes == 2 then
       --- We need to test for `encodingbytes` to avoid passing
       --- tfm fonts to `patch_font`. These fonts are fragile
       --- because they use traditional TeX font handling.
-      luatexbase.call_callback ("luaotfload.patch_font", fontdata, specification)
+      luatexbase.call_callback ("luaotfload.patch_font", fontdata, specification, id)
     else
-      luatexbase.call_callback ("luaotfload.patch_font_unsafe", fontdata, specification)
+      luatexbase.call_callback ("luaotfload.patch_font_unsafe", fontdata, specification, id)
     end
-    return fontdata
+    if not fontdata or id or tonumber(fontdata) then
+      return fontdata
+    end
+    id = font.define(fontdata)
+    register(fontdata, id)
+    return id
   end
 
   local mk_info = function (name)
@@ -218,26 +224,25 @@ local install_callbacks = function ()
   create_callback ("luaotfload.patch_font",        "simple", dummy_function)
   create_callback ("luaotfload.patch_font_unsafe", "simple", dummy_function)
   purge_define_font ()
-  local definer = config.luaotfload.run.definer
+  local definer = config.luaotfload.run.definer or "patch"
+  local selected_definer = definers[definer]
+  luaotfload.define_font = selected_definer
   luatexbase.add_to_callback ("define_font",
-                              definers[definer or "patch"],
+                              selected_definer,
                               "luaotfload.define_font",
                               1)
   return true
 end
 
-return {
-  init = function ()
-    local ret = true
-    if not install_formats () then
-      logreport ("log", 0, "loaders", "Error initializing OFM/PF{A,B} loaders.")
-      ret = false
-    end
-    if not install_callbacks () then
-      logreport ("log", 0, "loaders", "Error installing font loader callbacks.")
-      ret = false
-    end
-    return ret
+return function ()
+  if not install_formats () then
+    logreport ("log", 0, "loaders", "Error initializing OFM/PF{A,B} loaders.")
+    return false
   end
-}
+  if not install_callbacks () then
+    logreport ("log", 0, "loaders", "Error installing font loader callbacks.")
+    return false
+  end
+  return true
+end
 -- vim:tw=79:sw=2:ts=2:expandtab
