@@ -1,4 +1,4 @@
-# $Id: TLPDB.pm 51487 2019-06-27 00:51:49Z preining $
+# $Id: TLPDB.pm 53076 2019-12-10 06:20:44Z preining $
 # TeXLive::TLPDB.pm - tlpdb plain text database files.
 # Copyright 2007-2019 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
@@ -6,7 +6,7 @@
 
 package TeXLive::TLPDB;
 
-my $svnrev = '$Revision: 51487 $';
+my $svnrev = '$Revision: 53076 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -91,7 +91,7 @@ C<TeXLive::TLPDB> -- A database of TeX Live Packages
 =cut
 
 use TeXLive::TLConfig qw($CategoriesRegexp $DefaultCategory $InfraLocation
-      $DatabaseName $MetaCategoriesRegexp $Archive
+      $DatabaseName $DatabaseLocation $MetaCategoriesRegexp $Archive
       $DefaultCompressorFormat %Compressors $CompressorExtRegexp
       %TLPDBOptions %TLPDBSettings $ChecksumExtension
       $RelocPrefix $RelocTree);
@@ -148,7 +148,7 @@ sub new {
   } 
   if (defined($self->{'root'})) {
     my $nr_packages_read
-      = $self->from_file("$self->{'root'}/$InfraLocation/$DatabaseName",
+      = $self->from_file("$self->{'root'}/$DatabaseLocation",
         'verify' => $verify);
     if ($nr_packages_read == 0) {
       # that is bad, we didn't read anything, so return undef.
@@ -290,6 +290,10 @@ sub from_file {
     }
     if ($params{'media'}) {
       $media = $params{'media'};
+    } elsif (! -d $rootpath) {
+      # no point in going on if we don't even have a directory.
+      tlwarn("TLPDB: not a directory, not loading: $rootpath\n");
+      return 0;
     } elsif (-d "$rootpath/texmf-dist/web2c") {
       $media = 'local_uncompressed';
     } elsif (-d "$rootpath/texmf/web2c") { # older
@@ -300,7 +304,7 @@ sub from_file {
       $media = 'local_compressed';
     } else {
       # we cannot find the right type, return zero, hope people notice
-      tlwarn("Cannot determine type of tlpdb from $rootpath!\n");
+      tlwarn("TLPDB: Cannot determine type of tlpdb from $rootpath!\n");
       return 0;
     }
   }
@@ -355,8 +359,9 @@ sub from_file {
         # do nothing
       } else {
         unlink($tlpdbfile);
-        tldie(  "$0: TLPDB::from_file could not download $path;\n"
-              . "$0: maybe the repository setting should be changed.\n");
+        tldie(  "$0: TLPDB::from_file could not initialize from: $path\n"
+              . "$0: Maybe the repository setting should be changed.\n"
+              . "$0: More info: https://tug.org/texlive/acquire.html\n");
       }
     }
     # if we are still here, then either the xz version was downloaded
@@ -1200,7 +1205,7 @@ sub location {
     tlwarn("TLPDB: cannot get location of a virtual tlpdb\n");
     return 0;
   }
-  return "$self->{'root'}/$InfraLocation/$DatabaseName";
+  return "$self->{'root'}/$DatabaseLocation";
 }
 
 =pod
@@ -1820,17 +1825,24 @@ sub not_virtual_install_package {
         }
       }
       if (!$container) {
-        tlwarn("TLPDB: cannot find package $pkg.tar.$CompressorExtRegexp in $root/$Archive\n");
+        tlwarn("TLPDB: cannot find package $pkg.tar.$CompressorExtRegexp"
+               . " in $root/$Archive\n");
         return(0);
       }
     } elsif (&media eq 'NET') {
-      $container = "$root/$Archive/$pkg.tar." . $Compressors{$DefaultCompressorFormat}{'extension'};
+      $container = "$root/$Archive/$pkg.tar."
+                   . $Compressors{$DefaultCompressorFormat}{'extension'};
     }
-    debug("TLPDB::not_virtual_install_package: trying to install $container\n");
-    $self->_install_data ($container, $reloc, \@installfiles, $totlpdb, $tlpobj->containersize, $tlpobj->containerchecksum)
+    my $container_str = ref $container eq "ARRAY"
+                        ? "[" . join (" ", @$container) . "]" : $container;
+    ddebug("TLPDB::not_virtual_install_package: installing container: ",
+          $container_str, "\n");
+    $self->_install_data($container, $reloc, \@installfiles, $totlpdb,
+                         $tlpobj->containersize, $tlpobj->containerchecksum)
       || return(0);
-    # if we are installing from local_compressed or NET we have to fetch the respective
-    # source and doc packages $pkg.source and $pkg.doc and install them, too
+    # if we are installing from local_compressed or NET we have to fetch
+    # respective source and doc packages $pkg.source and $pkg.doc and
+    # install them, too
     if (($media eq 'NET') || ($media eq 'local_compressed')) {
       # we install split containers under the following conditions:
       # - the container were split generated
@@ -1846,13 +1858,15 @@ sub not_virtual_install_package {
       if ($container_src_split && $opt_src && $tlpobj->srcfiles) {
         my $srccontainer = $container;
         $srccontainer =~ s/\.tar\.$CompressorExtRegexp$/.source.tar.$1/;
-        $self->_install_data ($srccontainer, $reloc, \@installfiles, $totlpdb, $tlpobj->srccontainersize, $tlpobj->srccontainerchecksum)
+        $self->_install_data($srccontainer, $reloc, \@installfiles, $totlpdb,
+                      $tlpobj->srccontainersize, $tlpobj->srccontainerchecksum)
           || return(0);
       }
       if ($container_doc_split && $real_opt_doc && $tlpobj->docfiles) {
         my $doccontainer = $container;
         $doccontainer =~ s/\.tar\.$CompressorExtRegexp$/.doc.tar.$1/;
-        $self->_install_data ($doccontainer, $reloc, \@installfiles, $totlpdb, $tlpobj->doccontainersize, $tlpobj->doccontainerchecksum)
+        $self->_install_data($doccontainer, $reloc, \@installfiles,
+            $totlpdb, $tlpobj->doccontainersize, $tlpobj->doccontainerchecksum)
           || return(0);
       }
       #
